@@ -87,6 +87,7 @@
 #include <sys/types.h>
 #include <stdlib.h>
 #include <assert.h>
+#include <fcntl.h>
 
 #if defined(FD_SET_IN_SYS_SELECT_H)
 #  include <sys/select.h>
@@ -194,11 +195,7 @@ static int tcp_read(
   int fd)  /* I */
 
   {
-  long              i;
-  long              total = 0;
-  unsigned long     newsize;
-  void             *ptr;
-  int               done = 0;
+  int               i;
 #ifdef HAVE_POLL
 
   struct pollfd pollset;
@@ -280,67 +277,35 @@ static int tcp_read(
     return(-1);
     }
 
-  while (1)
+  while ((i = read(
+                fd,
+                tp->tdis_eod,
+                tp->tdis_thebuf + tp->tdis_bufsize - tp->tdis_eod)) == -1)
     {
-    while ((i = read(
-                  fd,
-                  tp->tdis_eod,
-                  tp->tdis_thebuf + tp->tdis_bufsize - tp->tdis_eod)) == -1)
-      {
-      if (errno != EINTR)
-        break;
-      }
-    
-    if (i < 0)
-      {
-      /* FAILURE - read failed */
-      
-      tcparray[fd]->ReadErrno = errno;
-      
-      return(-1);
-      }
-    else if (i == 0)
-      {
-      /* FAILURE - no data read */
-      
-      return(-2);
-      }
-    
-    /* SUCCESS */
-    if (i < tp->tdis_thebuf + tp->tdis_bufsize - tp->tdis_eod)
-      done = 1;
-    
-    tp->tdis_eod += i;
-    total += i;
-
-    /* done reading if we haven't read our max size */
-    if (done)
+    if (errno != EINTR)
       break;
-
-    /* otherwise alloc more space and read again, add 25% more space */
-    newsize = tp->tdis_bufsize * 1.25;
-    ptr = realloc(tp->tdis_thebuf,newsize);
-
-    if (ptr == NULL)
-      {
-      /* failure to allocate memory, return NULL? */
-      log_err(ENOMEM,"tcp_read","Could not allocate memory to read buffer");
-      return(-1);
-      }
-    else
-      {
-      /* adjust the new values */
-      tp->tdis_eod += ((char *)ptr) - tp->tdis_thebuf;
-      tp->tdis_trailp += ((char *)ptr) - tp->tdis_thebuf;
-      tp->tdis_leadp += ((char *)ptr) - tp->tdis_thebuf;
-      tp->tdis_thebuf = (char *)ptr;
-      tp->tdis_bufsize = newsize;
-
-      /* fall through to continue reading more */
-      }
     }
 
-  return(total);
+  if (i < 0)
+    {
+    /* FAILURE - read failed */
+
+    tcparray[fd]->ReadErrno = errno;
+
+    return(-1);
+    }
+  else if (i == 0)
+    {
+    /* FAILURE - no data read */
+
+    return(-2);
+    }
+
+  /* SUCCESS */
+
+  tp->tdis_eod += i;
+
+  return(i);
   }  /* END tcp_read() */
 
 
@@ -666,7 +631,7 @@ static int tcp_puts(
     /* not enough room, reallocate the buffer */
     leadpct = (int)(tp->tdis_thebuf - tp->tdis_leadp);
     trailpct = (int)(tp->tdis_thebuf - tp->tdis_trailp);
-    newbufsize = tp->tdis_bufsize + THE_BUF_SIZE + ct;
+    newbufsize = tp->tdis_bufsize + THE_BUF_SIZE;
     temp = (char *)malloc(newbufsize);
     if(!temp)
       {
@@ -824,6 +789,23 @@ void DIS_tcp_setup(
   if (fd >= tcparraymax)
     {
     int hold = tcparraymax;
+    int flags;
+    
+    /*
+    ** Check if this is a valid file descriptor, if not log an error and don't
+    ** do any memory allocations
+    */
+
+    flags = fcntl(fd, F_GETFL);
+
+    if (errno == EBADF)
+      {
+      sprintf(log_buffer, "invalid file descriptor (%d) for socket",
+        fd);
+      log_err(errno, "DIS_tcp_setup", log_buffer);
+
+      return;
+      }
 
     tcparraymax = fd + 10;
 
