@@ -99,7 +99,7 @@
 #include "mom_mach.h"
 #include "mom_func.h"
 
-static char ident[] = "@(#) $RCSfile$ $Revision: 4237 $";
+static char ident[] = "@(#) $RCSfile$ $Revision: 3230 $";
 
 /* Global Variables */
 
@@ -150,7 +150,7 @@ gid_t gid;
 
   for (i = 0; i < DIR_RETRY; i++)
     {
-    if (mkdir(name, 01777) == -1)
+    if (mkdir(name, 0777) == -1)
       {
       if (errno == EEXIST)
         {
@@ -202,7 +202,7 @@ mktmpdir(
   task *ptask;
   int tasks;
 #if SRFS
-  extern char *var_value A_((char *));
+  extern char *var_value(char *);
 #endif
 
   uid = pjob->ji_qs.ji_un.ji_momt.ji_exuid;
@@ -320,7 +320,7 @@ rmtmpdir(char *sequence)
   static char *id = "rmtmpdir";
   char *tmpdir;
   int pid, pstat;
-  extern char *var_value A_((char *));
+  extern char *var_value(char *);
 
   pid = fork();
 
