@@ -102,7 +102,7 @@
 #include "mom_mach.h"
 #include "mom_func.h"
 
-static char ident[] = "@(#) $RCSfile$ $Revision: 4237 $";
+static char ident[] = "@(#) $RCSfile$ $Revision: 3230 $";
 
 /* Global Variables */
 
@@ -153,7 +153,7 @@ gid_t gid;
 
   for (i = 0; i < DIR_RETRY; i++)
     {
-    if (mkdir(name, 01777) == -1)
+    if (mkdir(name, 0777) == -1)
       {
       if (errno == EEXIST)
         {
@@ -214,7 +214,7 @@ mktmpdir(
        ptask = (task *)GET_NEXT(ptask->ti_jobtask), tasks++);
 
 #if SRFS
-  extern char *var_value A_((char *));
+  extern char *var_value(char *);
 
   tmpdir = tmpdirname(var_value("TMPDIR"), seq);
 
@@ -322,7 +322,7 @@ rmtmpdir(char *sequence)
   static char *id = "rmtmpdir";
   char *tmpdir;
   int pid, pstat;
-  extern char *var_value A_((char *));
+  extern char *var_value(char *);
 
   pid = fork();
 
