@@ -529,7 +529,7 @@ int conn_qsub(
 
   if ((hostaddr = get_hostaddr(hostname)) == (pbs_net_t)0)
     {
-#if !defined(H_ERRNO_DECLARED)
+#if !defined(H_ERRNO_DECLARED) && !defined(_AIX)
     extern int h_errno;
 #endif
 
@@ -772,7 +772,6 @@ job *job_clone(
            taskid,
            hostname);
 
-  free(oldid);
 
   /* update the job filename
    * We could optimize the sub-jobs to all use the same file. We would need a
@@ -784,8 +783,8 @@ job *job_clone(
    * make up new job file name, it is based on the new jobid
    */
 
-  strncpy(basename, pnewjob->ji_qs.ji_jobid, PBS_JOBBASE);
-  basename[PBS_JOBBASE] = '\0';
+  snprintf(basename, PBS_JOBBASE, "%s-%d.%s", oldid, taskid, hostname);
+  free(oldid);
 
   do
     {
@@ -849,7 +848,7 @@ job *job_clone(
 
         tmpstr = (char*)malloc(sizeof(char) * (slen + PBS_MAXJOBARRAYLEN + 1));
 
-        sprintf(tmpstr, "%s[%d]",
+        sprintf(tmpstr, "%s-%d",
                 template_job->ji_wattr[i].at_val.at_str,
                 taskid);
 
@@ -1510,6 +1509,7 @@ void job_purge(
       {
       array_delete(pjob->ji_arraystruct);
       }
+    array_save(pa);
     }
 
   if ((pjob->ji_is_array_template == TRUE) ||
