@@ -124,11 +124,14 @@
 #include "net_connect.h"
 #include "utils.h"
 
-#if !defined(H_ERRNO_DECLARED)
+#if !defined(H_ERRNO_DECLARED) && !defined(_AIX)
 extern int h_errno;
 #endif
 extern time_t  time_now;
 
+#ifndef MAX_LINE
+#define MAX_LINE 1024
+#endif
 
 /* Global Data */
 
@@ -175,7 +178,7 @@ extern void ping_nodes(struct work_task *);
 
 
 
-/* us IP address to look up matchin node structure */
+/* use IP address to look up matchin node structure */
 
 struct pbsnode *PGetNodeFromAddr(
 
@@ -908,6 +911,7 @@ static int process_host_name_part(
 
   {
 
+  char id[] = "process_host_name_part";
   struct hostent *hp;
 
   struct in_addr  addr;
@@ -954,6 +958,8 @@ static int process_host_name_part(
     sprintf(log_buffer, "host %s not found",
             objname);
 
+    log_err(PBSE_UNKNODE, id, log_buffer);
+
     free(phostname);
     phostname = NULL;
 
@@ -994,6 +1000,8 @@ static int process_host_name_part(
               errno,
               pbs_strerror(errno));
 
+      log_err(PBSE_UNKNODE, id, log_buffer);
+
       free(phostname);
       phostname = NULL;
 
@@ -1073,6 +1081,8 @@ static int process_host_name_part(
                 errno,
                 pbs_strerror(errno));
 
+        log_err(PBSE_UNKNODE, id, log_buffer);
+
         if (hname != NULL)
           {
           free(hname);
@@ -1604,7 +1614,7 @@ static char *parse_node_token(
 
   for (;pt[0] != '\0';pt++)
     {
-    if (isalnum((int)*pt) || strchr("-._", *pt) || (*pt == '\0'))
+    if (isalnum((int)*pt) || strchr("-._[]", *pt) || (*pt == '\0'))
       continue;
 
     if (isspace((int)*pt))
@@ -1653,8 +1663,14 @@ int setup_nodes(void)
   char  *nodename;
   char   propstr[256];
   char  *token;
-  int   bad, i, num, linenum;
-  int   err;
+  char  *open_bracket;
+  char  *close_bracket;
+  char  *dash;
+  char   tmp_node_name[MAX_LINE];
+  int    bad, i, num, linenum;
+  int    err;
+  int    start = -1;
+  int    end = -1;
 
   struct pbsnode *np;
   char     *val;
@@ -1822,7 +1838,75 @@ int setup_nodes(void)
 
     pal = GET_NEXT(atrlist);
 
-    err = create_pbs_node(nodename, pal, perm, &bad);
+    if ((open_bracket = strchr(nodename,'[')) != NULL)
+      {
+      int num_digits;
+
+      start = atoi(open_bracket+1);
+      
+      dash = strchr(open_bracket,'-');
+      close_bracket = strchr(open_bracket,']');
+
+      if ((dash == NULL) ||
+          (close_bracket == NULL))
+        {
+        sprintf(log_buffer,
+          "malformed nodename with range: %s, must be of form [x-y]\n",
+          nodename);
+
+        log_err(-1,id,log_buffer);
+
+        goto errtoken2;
+        }
+
+      end = atoi(dash+1);
+
+      /* nullify the open bracket */
+      *open_bracket = '\0';
+
+      num_digits = dash - open_bracket - 1;
+
+      /* move past the closing bracket */
+      close_bracket++;
+
+      while (start <= end)
+        {
+        int num_len = 1;
+        int tmp = 10;
+
+        strcpy(tmp_node_name,nodename);
+       
+        /* determine the length of the number */
+        while (start / tmp > 0)
+          {
+          tmp *= 10;
+          num_len++;
+          }
+
+        /* print extra zeros if needed */
+        while (num_len < num_digits)
+          {
+          strcat(tmp_node_name,"0");
+
+          num_len++;
+          }
+
+        sprintf(tmp_node_name+strlen(tmp_node_name),"%d%s",
+          start,
+          close_bracket);
+
+        err = create_pbs_node(tmp_node_name,pal,perm,&bad);
+
+        if (err != 0)
+          break;
+
+        start++;
+        }
+      }
+    else
+      {
+      err = create_pbs_node(nodename, pal, perm, &bad);
+      }
 
     if (err == PBSE_NODEEXIST)
       {
