@@ -117,14 +117,6 @@
 #include <signal.h>
 /*#endif*/
 
-#ifndef SUCCESS
-#define SUCCESS 1
-#endif /* SUCCESS */
-
-#ifndef FAILURE
-#define FAILURE 0
-#endif /* FAILURE */
-
 #ifndef TRUE
 #define TRUE 1
 #endif /* TRUE */
@@ -180,6 +172,7 @@ extern char  server_name[];
 extern int  svr_delay_entry;
 extern tlist_head svr_newjobs;
 extern tlist_head svr_alljobs;
+extern tlist_head svr_jobs_array_sum;
 extern tlist_head svr_queues;
 extern tlist_head svr_requests;
 extern tlist_head svr_newnodes;
@@ -198,31 +191,31 @@ extern struct server server;
 
 /* External Functions Called */
 
-extern void   on_job_exit A_((struct work_task *));
-extern void   on_job_rerun A_((struct work_task *));
-extern void   set_resc_assigned A_((job *, enum batch_op));
-extern void   set_old_nodes A_((job *));
-extern void   acct_close A_((void));
+extern void   on_job_exit(struct work_task *);
+extern void   on_job_rerun(struct work_task *);
+extern void   set_resc_assigned(job *, enum batch_op);
+extern void   set_old_nodes(job *);
+extern void   acct_close(void);
 
-extern struct work_task *apply_job_delete_nanny A_((struct job *, int));
-extern int     net_move A_((job *, struct batch_request *));
-extern void  job_clone_wt A_((struct work_task *));
+extern struct work_task *apply_job_delete_nanny(struct job *, int);
+extern int     net_move(job *, struct batch_request *);
+extern void  job_clone_wt(struct work_task *);
 
 /* Private functions in this file */
 
-static void  init_abt_job A_((job *));
-static char *build_path A_((char *, char *, char *));
-static void  catch_child A_((int));
-static void  catch_abort A_((int));
-static void  change_logs A_((int));
-static void  change_log_level A_((int));
-static int   chk_save_file A_((char *));
-static void  need_y_response A_((int));
-static int   pbsd_init_job A_((job *, int));
-static void  pbsd_init_reque A_((job *, int));
-static void  resume_net_move A_((struct work_task *));
-static void  rm_files A_((char *));
-static void  stop_me A_((int));
+static void  init_abt_job(job *);
+static char *build_path(char *, char *, char *);
+static void  catch_child(int);
+static void  catch_abort(int);
+static void  change_logs(int);
+static void  change_log_level(int);
+static int   chk_save_file(char *);
+static void  need_y_response(int);
+static int   pbsd_init_job(job *, int);
+static void  pbsd_init_reque(job *, int);
+static void  resume_net_move(struct work_task *);
+static void  rm_files(char *);
+static void  stop_me(int);
 
 /* private data */
 
@@ -724,6 +717,8 @@ int pbsd_init(
 
   CLEAR_HEAD(svr_alljobs);
 
+  CLEAR_HEAD(svr_jobs_array_sum);
+
   CLEAR_HEAD(svr_newjobs);
 
   CLEAR_HEAD(svr_newnodes);
@@ -737,10 +732,10 @@ int pbsd_init(
   for (i = 0;i < SRV_ATR_LAST;i++)
     clear_attr(&server.sv_attr[i], &svr_attr_def[i]);
 
-  server.sv_attr[(int)SRV_ATR_schedule_iteration].at_val.at_long =
+  server.sv_attr[(int)SRV_ATR_scheduler_iteration].at_val.at_long =
     PBS_SCHEDULE_CYCLE;
 
-  server.sv_attr[(int)SRV_ATR_schedule_iteration].at_flags =
+  server.sv_attr[(int)SRV_ATR_scheduler_iteration].at_flags =
     ATR_VFLAG_SET;
 
   server.sv_attr[(int)SRV_ATR_State].at_val.at_long = SV_STATE_INIT;
@@ -979,10 +974,10 @@ int pbsd_init(
 
         if (pa == NULL)
           {
-          /* TODO GB */
 
           sprintf(log_buffer,
-                  "could not recover array-struct from file %s--skipping",
+                  "could not recover array-struct from file %s--skipping. "
+                  "job array can not be recovered.",
                   pdirent->d_name);
 
           log_err(errno, "pbsd_init", log_buffer);
@@ -1064,13 +1059,13 @@ int pbsd_init(
           {
           if ((pjob = job_recov(pdirent->d_name)) != NULL)
             {
-            append_link(&svr_alljobs, &pjob->ji_alljobs, pjob);
-            pjob->ji_isparent = TRUE;
-            }
-          else
-            {
-            /* FIXME: what should we do here?  we won't be able to finish 
-               cloning this array because the initial job file is missing */
+            pjob->ji_is_array_template = TRUE;
+            
+            if (DArrayAppend(&Array,pjob) == FAILURE)
+              {
+              log_err(ENOMEM,"main","out of memory reloading jobs");
+              exit(-1);
+              }
             }
 
           continue;
@@ -1198,42 +1193,54 @@ int pbsd_init(
       }
     }
 
-  /* look for empty arrays and delete them
+  /* finish setting up array structs
+     look for empty arrays and delete them
      also look for arrays that weren't fully built and setup a work task to
      continue the cloning process*/
   pa = (job_array*)GET_NEXT(svr_jobarrays);
 
   while (pa != NULL)
     {
-    if (pa->ai_qs.num_cloned != pa->ai_qs.array_size)
+    pa->template_job = find_array_template(pa->ai_qs.parent_id);
+    
+    if (pa->ai_qs.num_cloned != pa->ai_qs.num_jobs)
       {
 
-      job *pjob = find_job(pa->ai_qs.parent_id);
-
-      if (pjob == NULL)
+      /* if we can't finish building the job array then delete whats been done 
+         so far */
+      if (pa->template_job == NULL)
         {
-        /* TODO, we need to so something here, we can't finish cloning the 
-           array! */
+        int i;
+        job_array *temp;
+        for (i = 0; i < pa->ai_qs.array_size; i++)
+          {
+          if (pa->jobs[i] != NULL)
+            {
+            job_purge(pa->jobs[i]);
+            }
+          }
 
+        temp = (job_array*)GET_NEXT(pa->all_arrays);
+        array_delete(pa);
+        pa = temp;
+        continue;
         }
       else
         {
-        /* TODO if num_cloned != num_recovered then something strange happend
-           it is possible num_recovered == num_cloned+1.  That means that the 
-           server terminated after cloning a job but before updating the saved 
-           array_info struct. we probably should delete that last job and start
-           the cloning process off at num_cloned. Someone must have been 
-           naughty and did a kill -9 on pbs_server  */
-        wt = set_task(WORK_Timed, time_now + 1, job_clone_wt, (void*)pjob);
+        /* TODO Someone must have been naughty and did a kill -9 on pbs_server, 
+           we might need to validate that the last job was fully initialized 
+           before continuing the cloning process. */
+        wt = set_task(WORK_Timed, time_now + 1, job_clone_wt, (void*)pa->template_job);
 
         }
 
       }
-    else if (GET_NEXT(pa->array_alljobs) == pa->array_alljobs.ll_struct)
+    else if (pa->ai_qs.jobs_done == pa->ai_qs.num_jobs && pa->template_job == NULL)
       {
       job_array *temp = (job_array*)GET_NEXT(pa->all_arrays);
       array_delete(pa);
       pa = temp;
+      continue;
       }
 
     if (pa != NULL)
@@ -1518,6 +1525,8 @@ static int pbsd_init_job(
     case JOB_SUBSTATE_WAITING:
 
     case JOB_SUBSTATE_PRERUN:
+    
+    case JOB_SUBSTATE_ARRAY_TEMP:
 
       pbsd_init_reque(pjob, CHANGE_STATE);
 
@@ -1592,6 +1601,14 @@ static int pbsd_init_job(
 
       pbsd_init_reque(pjob, KEEP_STATE);
 
+      /* do array bookeeping */
+      if ((pjob->ji_arraystruct != NULL) &&
+          (pjob->ji_is_array_template == FALSE))
+        {
+        update_array_values(pjob->ji_arraystruct,
+          pjob,JOB_STATE_RUNNING,aeTerminate);
+        }
+
       break;
 
     case JOB_SUBSTATE_RERUN:
@@ -1832,7 +1849,7 @@ static void catch_child(
         msg_daemonname,
         log_buffer);
 #else
-      DBPRT(("catch_child caught pid %d\n", (int)pid));
+      DBPRT(("catch_child caught pid %d\n", pid));
 #endif
       }
 
@@ -1863,7 +1880,7 @@ static void catch_child(
             msg_daemonname,
             log_buffer);
 #else
-          DBPRT(("catch_child found work task found for pid %d\n", (int)pid));
+          DBPRT(("catch_child found work task found for pid %d\n", pid));
 #endif
           }
 
@@ -1880,7 +1897,7 @@ static void catch_child(
 
       log_err(-1, "catch_child", log_buffer);
 #else
-      DBPRT(("catch_child no work task found for pid %d\n", (int)pid));
+      DBPRT(("catch_child no work task found for pid %d\n", pid));
 #endif
       }
     }    /* END while (1) */
