@@ -90,7 +90,6 @@
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
-#include <errno.h>
 #include "libpbs.h"
 #include "server_limits.h"
 #include "list_link.h"
@@ -108,6 +107,7 @@
 #include "svrfunc.h"
 #include "sched_cmds.h"
 #include "queue.h"
+#include "array.h"
 
 
 #define RESC_USED_BUF 2048
@@ -130,7 +130,6 @@ extern char *msg_obitnocpy;
 extern char *msg_obitnodel;
 extern char  server_host[];
 extern int   svr_do_schedule;
-extern int   listener_command;
 extern time_t time_now;
 
 extern int   LOGLEVEL;
@@ -139,12 +138,12 @@ extern const char *PJobState[];
 
 /* External Functions called */
 
-extern void set_resc_assigned A_((job *, enum batch_op));
+extern void set_resc_assigned(job *, enum batch_op);
 extern void cleanup_restart_file(job *);
 
 /* Local public functions  */
 
-void req_jobobit A_((struct batch_request *));
+void req_jobobit(struct batch_request *);
 
 
 
@@ -344,8 +343,8 @@ static struct batch_request *return_stdfile(
 
   {
 
-  if (pjob->ji_wattr[(int)JOB_ATR_interactive].at_flags &&
-      pjob->ji_wattr[(int)JOB_ATR_interactive].at_val.at_long)
+  if ((pjob->ji_wattr[(int)JOB_ATR_interactive].at_flags) &&
+      (pjob->ji_wattr[(int)JOB_ATR_interactive].at_val.at_long))
     {
     return NULL;
     }
@@ -403,8 +402,8 @@ static struct batch_request *cpy_stdfile(
   char *suffix;
   char *to = NULL;
 
-  if (pjob->ji_wattr[(int)JOB_ATR_interactive].at_flags &&
-      pjob->ji_wattr[(int)JOB_ATR_interactive].at_val.at_long)
+  if ((pjob->ji_wattr[(int)JOB_ATR_interactive].at_flags) &&
+      (pjob->ji_wattr[(int)JOB_ATR_interactive].at_val.at_long))
     {
     /* the job is interactive, don't bother to return output file */
 
@@ -634,7 +633,7 @@ struct batch_request *cpy_stage(
 int mom_comm(
 
   job *pjob,
-  void (*func) A_((struct work_task *)))
+  void (*func)(struct work_task *))
 
   {
   unsigned int dummy;
@@ -713,7 +712,6 @@ void rel_resc(
   /* mark that scheduler should be called */
 
   svr_do_schedule = SCH_SCHEDULE_TERM;
-  listener_command = SCH_SCHEDULE_TERM;
 
   return;
   }  /* END rel_resc() */
@@ -746,7 +744,9 @@ void on_job_exit(
   {
   int    handle = -1;
   job   *pjob;
+#ifdef VNODETESTING
   job   *pj;
+#endif
 
   struct batch_request *preq;
 
@@ -755,7 +755,6 @@ void on_job_exit(
   int  PurgeIt = FALSE;
   int MustReport = FALSE;
   pbs_queue *pque;
-  char *id = "on_job_exit";
   char namebuf[MAXPATHLEN + 1];
   char *namebuf2;
   int spool_file_exists;
@@ -776,7 +775,9 @@ void on_job_exit(
     pjob = (job *)preq->rq_extra;
     }
 
-  /* make sure the job is actually still there */
+#ifdef VNODETESTING
+  /* FIXME: there might be a race with calling on_job_exit after a job has
+   * already been free'd.  This is temp code */
 
   pj = (job *)GET_NEXT(svr_alljobs);
 
@@ -788,13 +789,28 @@ void on_job_exit(
     pj = (job *)GET_NEXT(pj->ji_alljobs);
     }
 
-  /* if the job doesn't exist, just exit */
   if (pj == NULL)
     {
-    log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,"NULL","processing exit for a non-existent job. Skiping");
-
-    return;
+    sprintf(log_buffer, "on_job_exit called with INVALID pjob: %p",
+            pjob);
     }
+  else
+    {
+    sprintf(log_buffer, "on_job_exit valid pjob: %p (substate=%d)",
+            pjob,
+            pjob->ji_qs.ji_substate);
+    }
+
+  log_event(
+
+    PBSEVENT_JOB,
+    PBS_EVENTCLASS_JOB,
+    pjob->ji_qs.ji_jobid,
+    log_buffer);
+
+  DBPRT(("%s\n",
+         log_buffer));
+#endif /* END VNODETESTING */
 
   /*
    * we don't need a handle if we are complete. On starting up we will NOT have
@@ -894,9 +910,13 @@ void on_job_exit(
             {
             strcpy(namebuf2, namebuf);
             strcat(namebuf2, ".SAV");
-            if (link(namebuf, namebuf2))
+            if (link(namebuf, namebuf2) == -1)
               {
-              log_err(errno,id,strerror(errno));
+              LOG_EVENT(
+                PBSEVENT_ERROR | PBSEVENT_SECURITY,
+                PBS_EVENTCLASS_JOB,
+                pjob->ji_qs.ji_jobid,
+                "Link(1) in on_job_exit failed");
               }
             }
 
@@ -914,9 +934,13 @@ void on_job_exit(
             {
             strcpy(namebuf2, namebuf);
             strcat(namebuf2, ".SAV");
-            if (link(namebuf, namebuf2))
+            if (link(namebuf, namebuf2) == -1)
               {
-              log_err(errno,id,strerror(errno));
+              LOG_EVENT(
+                PBSEVENT_ERROR | PBSEVENT_SECURITY,
+                PBS_EVENTCLASS_JOB,
+                pjob->ji_qs.ji_jobid,
+                "Link(2) in on_job_exit failed");
               }
             }
 
@@ -1203,9 +1227,13 @@ void on_job_exit(
 
       if (spool_file_exists == 0)
         {
-        if (link(namebuf2, namebuf))
+        if (link(namebuf2, namebuf) == -1)
           {
-          log_err(errno,id,strerror(errno));
+          LOG_EVENT(
+            PBSEVENT_ERROR | PBSEVENT_SECURITY,
+            PBS_EVENTCLASS_JOB,
+            pjob->ji_qs.ji_jobid,
+            "Link(3) in on_job_exit failed");
           }
         unlink(namebuf2);
         }
@@ -1223,9 +1251,13 @@ void on_job_exit(
       if (spool_file_exists == 0)
         {
 
-        if (link(namebuf2, namebuf))
+        if (link(namebuf2, namebuf) == -1)
           {
-          log_err(errno,id,strerror(errno));
+          LOG_EVENT(
+            PBSEVENT_ERROR | PBSEVENT_SECURITY,
+            PBS_EVENTCLASS_JOB,
+            pjob->ji_qs.ji_jobid,
+            "Link(4) in on_job_exit failed");
           }
         unlink(namebuf2);
         }
@@ -2623,6 +2655,14 @@ void req_jobobit(
 
     ptask = set_task(WORK_Immed, 0, on_job_exit, (void *)pjob);
 
+    /* decrease array running job count */
+    if ((pjob->ji_arraystruct != NULL) &&
+        (pjob->ji_is_array_template == FALSE))
+      {
+      update_array_values(pjob->ji_arraystruct,
+        pjob,JOB_STATE_RUNNING,aeTerminate);
+      }
+
     if (ptask != NULL)
       {
       append_link(&pjob->ji_svrtask, &ptask->wt_linkobj, ptask);
