@@ -106,8 +106,8 @@
 #include "resource.h"
 
 
-extern int     svr_authorize_jobreq A_((struct batch_request *, job *));
-int status_attrib A_((svrattrl *, attribute_def *, attribute *, int, int, tlist_head *, int *, int));
+extern int     svr_authorize_jobreq(struct batch_request *, job *);
+int status_attrib(svrattrl *, attribute_def *, attribute *, int, int, tlist_head *, int *, int);
 
 /* Global Data Items: */
 
@@ -288,8 +288,7 @@ int status_attrib(
           ATR_ENCODE_CLIENT);
 
         /* add walltime remaining if started */
-        if ((index == JOB_ATR_start_time) &&
-            ((pattr + index)->at_flags & ATR_VFLAG_SET))
+        if (index == JOB_ATR_start_time)
           {
           /* encode walltime remaining, this is custom because walltime 
            * remaining isn't an attribute */
@@ -300,8 +299,8 @@ int status_attrib(
           resource *pres;
 
           int found = 0;
-          unsigned long remaining;
-          unsigned long upperBound;
+          unsigned long remaining = 0;
+          unsigned long upperBound = 0;
 
           if (((pattr + JOB_ATR_resource)->at_val.at_list.ll_next != NULL) &&
               ((pattr + JOB_ATR_resource)->at_flags & ATR_VFLAG_SET))
@@ -316,35 +315,28 @@ int status_attrib(
               {
               upperBound = (unsigned long)time_now;
               }
-            
+
             /* find the walltime resource */
             for (;pres != NULL;pres = (resource *)GET_NEXT(pres->rs_link))
               {
               pname = pres->rs_defin->rs_name;
-              
+
               if (strcmp(pname, "walltime") == 0)
                 {
                 /* found walltime */
                 unsigned long value = (unsigned long)pres->rs_value.at_val.at_long;
-                if (((pattr+index)->at_flags & ATR_VFLAG_SET) != FALSE)
-                  {
-                  remaining = value - (upperBound - (pattr + index)->at_val.at_long);
-                  }
-                else
-                  {
-                  remaining = value;
-                  }
+                remaining = value - (time_now - (pattr + index)->at_val.at_long);
                 found = TRUE;
                 break;
                 }
               }
             }
-         
+
           if (found == TRUE)
             {
-            snprintf(buf,sizeof(buf),"%lu",remaining);
+            snprintf(buf,sizeof(buf),"%ld",remaining);
 
-            len = strlen(buf) + 1;
+            len = strlen(buf+1);
             pal = attrlist_create("Walltime","Remaining",len);
 
             if (pal != NULL)
