@@ -369,9 +369,9 @@ int svr_enquejob(
       {
       /* link after 'current' job in server's list */
 
-      /* only link if it isn't the same job - it may already be inserted
-       * if submitted to a routing queue */
-      if (strcmp(pjcur->ji_qs.ji_jobid,pjob->ji_qs.ji_jobid))
+      /* only link if it isn't already inserted. routing queues, 
+       * remapping, etc. can mean it was already inserted */
+      if (!is_link_initialized(&pjob->ji_jobs_array_sum))
         {
         insert_link(&pjcur->ji_jobs_array_sum, &pjob->ji_jobs_array_sum,
           pjob,LINK_INSET_AFTER);
